/*
  Copyright 2008 CNRS
  Author: Florent Lamiraux
*/

#ifndef HPPWALKSTEERINGMETHOD_H
#define HPPWALKSTEERINGMETHOD_H

#include "KineoWorks2/kwsSteeringMethod.h"

KIT_PREDEF_CLASS(CkwsDirectPath);
KIT_PREDEF_CLASS(ChppWalkSteeringMethod);

/**
   \brief Steering method for humanoid walk motion

   This steering method is intended to be used to plan paths for 
   the bounding box of a humanoid robot. 
   \li first the steering method tries to build a CflicDirectPath between
   the initial and goal configuration. In case of success, the direct path is returned.
   \li then if the distance between the initial and goal configurations is below a threshold,
   the linear direct path is returned (CkwsPlusDPLinear),
   \li otherwise, the steering method returns no direct path.
*/
   
class ChppWalkSteeringMethod : public CkwsSteeringMethod
{
public:
  /**
     \brief Destructor
  */
  virtual ~ChppWalkSteeringMethod();

  /**
     \brief Create a steering method and returns a shared pointer

     \param inDistanceThreshold distance below which a linear direct path can be 
     inserted between the configurations.

  */
  static ChppWalkSteeringMethodShPtr create(double inDistanceThreshold);
  /**
     \brief  Factory method that creates a new direct path between two configurations.
     \param inStartCfg : the start config
     \param inEndCfg : the end config
     \return A shared pointer on the direct path created
  */
  virtual CkwsDirectPathShPtr makeDirectPath(const CkwsConfig& inStartCfg, 
					     const CkwsConfig& inEndCfg);
 	
  /**
     \brief  Returns whether the steering method produces oriented or non-oriented paths.
     \return true if the steering method is oriened or false
  */
  virtual bool isOriented() const {
    return true;
  };

 protected:

  /** 
      \brief Constructor.
  */
  ChppWalkSteeringMethod(double inDistanceThreshold);

  /** 
      \brief Initialization 
      \param         inSteeringMethodWkPtr : weak pointer to the object itself
      \return        KD_OK or KD_ERROR
  */
  ktStatus init(const CkwsSteeringMethodWkPtr& inSteeringMethodWkPtr);

 	
 private:

  /**
     \brief Weak pointer to itself

     Passed to direct paths
  */
  CkwsSteeringMethodWkPtr   attWeakPtr;

  /**
     \brief Distance function
  */
  CkwsDistanceShPtr attDistance;

  /**
     \brief Distance above which linear direct path is not allowed
  */
  double attDistanceThreshold;
};

#endif
