/*
  Copyright 2008 CNRS-AIST

  Author Florent Lamiraux
*/

#ifndef HPPWALKSTEPSEQUENCE_H
#define HPPWALKSTEPSEQUENCE_H

#include <iostream>
#include <vector>

/**
   \brief Step sequence for a walk motion
*/
class ChppWalkStepSequence {
public:
  /**
     \brief Constructor by initial position in global frame
  */
  ChppWalkStepSequence() : 
    attLeftFootFirst(true), attXInit(0), attYInit(0), attThetaInit(0),
    attStepSequence() {};

  /**
     \brief Structure for a relative foot print
     \param sx : abscissa of the landing foot in the local frame of the support foot.
     \param sy : ordinate of the landing foot in the local frame of the support foot.
     \param theta : orientation of the landing foot in the local frame of the support foot.
  */
  typedef struct TfootPrint {
    double sx ;
    double sy ;
    double theta ;
  } TfootPrint ;

  /**
     \brief return whether step sequence starts from left foot
  */
  bool leftFootFirst() const {
    return attLeftFootFirst;
  };

  /**
     \brief Set whether step sequence starts from left foot
  */
  void leftFootFirst(bool inLeftFootFirst) {
    attLeftFootFirst = inLeftFootFirst;
  };

  /**
     \brief Set initial position

     \param inXInit, inYInit, inZInit, inThetaInit position of the robot in global frame at the beginning of the motion.
  */
  void setInitialPosition(double inXInit, double inYInit, double inZInit, double inThetaInit) {
    attXInit = inXInit;
    attYInit = inYInit;
    attZInit = inZInit;
    attThetaInit = inThetaInit;
  };

  /**
     \brief Get initial position
     \retval outXinit, outYinit, outZinit, outThetaInit position of the robot in global frame at the beginning of the motion.
  */
  void getInitialPosition(double& outXInit, double& outYInit, double& outZInit, double& outThetaInit) const {
    outXInit = attXInit;
    outYInit = attYInit;
    outZInit = attZInit;
    outThetaInit = attThetaInit;
  };

  /**
     \brief Access to steps in the sequence
  */
  TfootPrint& operator[](unsigned int inIndex) {
    return attStepSequence[inIndex];
  };

  /**
     \brief Read only access to steps in the sequence
  */
  const TfootPrint& operator[](unsigned int inIndex) const {
    return attStepSequence[inIndex];
  };

  /**
     \brief Reinitialize step sequence
  */
  void clear() {
    attStepSequence.clear();
  };

  /**
     \brief Add a step at the end of the sequence
  */
  void push_back(const TfootPrint& inFootPrint) {
    attStepSequence.push_back(inFootPrint);
  };

  /**
     \brief Size of the step sequence
  */
  unsigned int size() const {
    return attStepSequence.size();
  };

private:

  /**
     \brief whether step sequence starts from left foot
  */
  bool attLeftFootFirst;
  
  /**
     \brief Initial position of the robot in global frame
     
     Step sequences are described locally in the local frame of the robot. 
     We therefore need to store the initial position of the robot.
  */
  double attXInit, attYInit, attZInit, attThetaInit;

  /**
     \brief vector of foot steps
     
     It is assumed that right and left feet are successively used (regular walking).
  */
  std::vector<TfootPrint> attStepSequence;
};

#if DEBUG>=2
std::ostream& operator<<(std::ostream& os, const ChppWalkStepSequence& inStepSequence);
#endif

#endif
